<?php

declare(strict_types = 1);

namespace Drupal\crema;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\Core\DependencyInjection\ServiceProviderBase;
use Drupal\Core\DrupalKernelInterface;
use Drupal\Core\Extension\InfoParserInterface;

/**
 * Service provider of Crema.
 *
 * Discovers replaced and replacement classes, then creates and registers
 * their special class loader.
 */
class CremaServiceProvider extends ServiceProviderBase {

  /**
   * {@inheritdoc}
   */
  public function alter(ContainerBuilder $container): void {
    $kernel = $container->get('kernel');
    assert($kernel instanceof DrupalKernelInterface);
    $modules = $container->getParameter('container.modules');
    $info_parser = $container->get('info_parser');
    assert($info_parser instanceof InfoParserInterface);
    $all_replacements = array_reduce(
      $modules,
      function (array $carry, array $module_info) use ($info_parser) {
        $info = $info_parser->parse($module_info['pathname']);
        if (empty($replacements = ($info['class_replacements'] ?? []))) {
          return $carry;
        }
        $dir = dirname($module_info['pathname']);
        array_walk(
          $replacements,
          function (&$relative_path, $key, $extension_dir) {
            $relative_path = $extension_dir . DIRECTORY_SEPARATOR . $relative_path;
          },
          $dir
        );
        $carry = array_merge($carry, $replacements);
        return $carry;
      },
      []
    );
    $app_root = $kernel->getAppRoot();
    $originals = array_reduce(
      array_keys($all_replacements),
      function (array $carry, string $replaced) use ($modules, $app_root) {
        if (preg_match('/^Drupal\\\([^\\\]*)\\\/', $replaced, $matches)) {
          if (isset($modules[$matches[1]])) {
            $dir = dirname($modules[$matches[1]]['pathname']);
            $carry["Drupal\\" . CremaClassLoader::ORIGINAL_SUBNAMESPACE . "\\{$matches[1]}"] = $app_root . DIRECTORY_SEPARATOR . $dir . '/src';
          }
        }
        return $carry;
      },
      []
    );
    $replacement_loader = new CremaClassLoader($all_replacements, $originals);
    $replacement_loader->register(TRUE);
  }

}
