<?php

namespace Drupal\Tests\crema\Unit;

use Drupal\crema\ClassCamouflage;
use Drupal\Tests\UnitTestCase;

/**
 * Tests ClassCamouflage.
 *
 * @coversDefaultClass \Drupal\crema\ClassCamouflage;
 *
 * @group crema
 */
class ClassCamouflageTest extends UnitTestCase {

  /**
   * Tests camouflage classes.
   *
   * @dataProvider camouflagedProvider
   */
  public function testGetCamouflaged(string $file_content, string $expected): void {
    $classCamouflage = new ClassCamouflage($file_content);
    $this->assertSame(
      $expected,
      $classCamouflage->getCamouflaged()
    );
  }

  /**
   * Data provider for ::testGetCamouflaged.
   *
   * @return string[][]
   *   The test cases.
   */
  public function camouflagedProvider(): array {
    return [
      'Simple class' => [
        'class' => <<<'EOL'
<?php

namespace Drupal\Foo;

abstract class FooBarBase extends FooBase implements FooInterface, BarInterface {

  use FooTrait;

}
EOL
        ,
        'expected' => <<<'EOL'
<?php

namespace Drupal\_original_\Foo;

use Drupal\Foo\FooBase;
use Drupal\Foo\FooInterface;
use Drupal\Foo\BarInterface;
use Drupal\Foo\FooTrait;

abstract class FooBarBase extends FooBase implements FooInterface, BarInterface {

  use FooTrait;

}
EOL
        ,
      ],
      'A bit more complicated class' => [
        'class' => <<<'EOL'
<?php

namespace Drupal\foo\Bar;

use Drupal\baz\BazTrait as BazBaseTrait;
use Drupal\foo\BarInterface;
use Drupal\foo\FooBar as FooBarBaz;

/**
 * Test class without any use statements.
 *
 * @see \Drupal\foo\Bar\FooBarBaz
 */
final class Baz extends FooBarBaz implements FooBazInterface, BarInterface {

  use BazBaseTrait {
    setup as public traitSetup;
  }
  use BazExtraLogicTrait;

  public static function fooBarBaz(self $self, parent $parent, array $array, callable $callable, bool $bool, float $float, int $int, string $string, iterable $iterable, object $object, mixed $mixed, int|float|bool|null $union): bool {
    return FALSE;
  }

  use FooFixtureTrait;

}

EOL
        ,
        'actual' => <<<'EOL'
<?php

namespace Drupal\_original_\foo\Bar;

use Drupal\foo\Bar\FooBazInterface;
use Drupal\foo\Bar\BazExtraLogicTrait;
use Drupal\foo\Bar\FooFixtureTrait;
use Drupal\baz\BazTrait as BazBaseTrait;
use Drupal\foo\BarInterface;
use Drupal\foo\FooBar as FooBarBaz;

final class Baz extends FooBarBaz implements FooBazInterface, BarInterface {

  use BazBaseTrait {
    setup as public traitSetup;
  }
  use BazExtraLogicTrait;

  public static function fooBarBaz(self $self, parent $parent, array $array, callable $callable, bool $bool, float $float, int $int, string $string, iterable $iterable, object $object, mixed $mixed, int|float|bool|null $union): bool {
    return FALSE;
  }

  use FooFixtureTrait;

}

EOL
        ,
      ],
      'A complex class' => [
        'class' => <<<'EOL'
<?php

namespace Drupal\foo\Bar;

use Drupal\foo\BarInterface;
use Drupal\foo\FooBar as FooBarAlias;

if (class_exists(FooBazInterface::class)) {
  class Baz extends FooBarAlias implements BarInterface, FooBazInterface, InterfaceInSameNs {

    use TraitInSameNs;

  }
}
else {
  class Baz extends FooBarAlias implements BarInterface, InterfaceInSameNs {

    use TraitInSameNs;
    use AnotherTraitInSameNs {
      foo as antotherTraitFoo;
    } 
    use YetAntotherTrait;

  }
}
EOL
        ,
        'actual' => <<<'EOL'
<?php

namespace Drupal\_original_\foo\Bar;

use Drupal\foo\Bar\FooBazInterface;
use Drupal\foo\Bar\InterfaceInSameNs;
use Drupal\foo\Bar\TraitInSameNs;
use Drupal\foo\Bar\AnotherTraitInSameNs;
use Drupal\foo\Bar\YetAntotherTrait;
use Drupal\foo\BarInterface;
use Drupal\foo\FooBar as FooBarAlias;

if (class_exists(FooBazInterface::class)) {
  class Baz extends FooBarAlias implements BarInterface, FooBazInterface, InterfaceInSameNs {

    use TraitInSameNs;

  }
}
else {
  class Baz extends FooBarAlias implements BarInterface, InterfaceInSameNs {

    use TraitInSameNs;
    use AnotherTraitInSameNs {
      foo as antotherTraitFoo;
    } 
    use YetAntotherTrait;

  }
}
EOL
        ,
      ],
      'A very complex class' => [
        'class' => <<<'EOL'
<?php

namespace Drupal\foo\Bar;

use Drupal\foo\BarInterface;
use Drupal\foo\FooBar as FooBarAlias;

if (class_exists(FooBazBarInterface::class)) {
  class Baz extends FooBarAlias implements BarInterface, FooBazInterface, InterfaceInSameNs {

    use TraitInSameNs;

    public function superFoo(array $foo, string $bar, AnotherUnusedClass $baz, YetAnother1|YetAnother2|iterable $union): array {
      return array_filter(
        $foo,
        function ($foo_value) use ($bar, $baz) {
          if ($baz[0] instanceof YetAnotherUnusedInterface) {
            return TRUE;
          }
          return $foo_value === $bar;
        }
      );
    }

  }
}
else {
  class Baz extends FooBarAlias implements BarInterface, InterfaceInSameNs {

    use TraitInSameNs;
    use AnotherTraitInSameNs {
      foo as antotherTraitFoo;
    } 
    use YetAntotherTrait;

  }
}
EOL
        ,
        'actual' => <<<'EOL'
<?php

namespace Drupal\_original_\foo\Bar;

use Drupal\foo\Bar\FooBazInterface;
use Drupal\foo\Bar\InterfaceInSameNs;
use Drupal\foo\Bar\TraitInSameNs;
use Drupal\foo\Bar\AnotherTraitInSameNs;
use Drupal\foo\Bar\YetAntotherTrait;
use Drupal\foo\Bar\FooBazBarInterface;
use Drupal\foo\Bar\AnotherUnusedClass;
use Drupal\foo\Bar\YetAnother1;
use Drupal\foo\Bar\YetAnother2;
use Drupal\foo\Bar\YetAnotherUnusedInterface;
use Drupal\foo\BarInterface;
use Drupal\foo\FooBar as FooBarAlias;

if (class_exists(FooBazBarInterface::class)) {
  class Baz extends FooBarAlias implements BarInterface, FooBazInterface, InterfaceInSameNs {

    use TraitInSameNs;

    public function superFoo(array $foo, string $bar, AnotherUnusedClass $baz, YetAnother1|YetAnother2|iterable $union): array {
      return array_filter(
        $foo,
        function ($foo_value) use ($bar, $baz) {
          if ($baz[0] instanceof YetAnotherUnusedInterface) {
            return TRUE;
          }
          return $foo_value === $bar;
        }
      );
    }

  }
}
else {
  class Baz extends FooBarAlias implements BarInterface, InterfaceInSameNs {

    use TraitInSameNs;
    use AnotherTraitInSameNs {
      foo as antotherTraitFoo;
    } 
    use YetAntotherTrait;

  }
}
EOL
        ,
      ],
    ];
  }

  /**
   * Tests new classes.
   *
   * @dataProvider newProvider
   */
  public function testGetNew(string $file_content, string $expected): void {
    $classCamouflage = new ClassCamouflage($file_content);
    $this->assertSame(
      $expected,
      $classCamouflage->getNew()
    );
  }

  /**
   * Data provider for ::testGetNew.
   *
   * @return string[][]
   *   The test cases.
   */
  public function newProvider(): array {
    return [
      'Simple class' => [
        'class' => <<<'EOL'
<?php

namespace Crema\migrate\Plugin;

use Drupal\migrate\Plugin\MigrationPluginManager as Original;

/**
 * Plugin manager for migration plugins.
 */
class MigrationPluginManager extends Original {

  /**
   * @return string
   */
  public function foo(): string {
    return 'foo';
  }

}
EOL
        ,
        'expected' => <<<'EOL'
<?php

namespace Drupal\migrate\Plugin;

use Drupal\_original_\migrate\Plugin\MigrationPluginManager as Original;

class MigrationPluginManager extends Original {

  public function foo(): string {
    return 'foo';
  }

}
EOL
        ,
      ],
    ];
  }

}
